import 'package:flutter/material.dart';
import '../marker_data.dart';
import '../consts.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:latlong2/latlong.dart';
import 'package:fl_chart/fl_chart.dart';

class MapContainerWidget extends StatefulWidget {
  final List<MarkerTemplate> markerList;

  const MapContainerWidget({Key? key, required this.markerList}) : super(key: key);

  @override
  _MapContainerWidgetState createState() => _MapContainerWidgetState();
}

class _MapContainerWidgetState extends State<MapContainerWidget> {

  MarkerTemplate? selectedMarker;
  bool isMinimized = true; // Quick view box state tacker

  @override
  Widget build(BuildContext context) {

    // NB: temporary test data
    final List<FlSpot> chartData = [
      FlSpot(0, 10),
      FlSpot(1, 20),
      FlSpot(2, 30),
      FlSpot(3, 40),
    ];
    double barWidth = 30; // Width for bars in bar chart
    const double contPadding = 30; // Container padding space

    return LayoutBuilder(
      builder: (context, constraints) {
        double screenWidth = constraints.maxWidth;
        double boxWidth = 0.86;
        double boxHeight = 1.4;

        return Column(
          children: [
            const SizedBox(height: contPadding),
            ClipRRect(
              borderRadius: BorderRadius.circular(20),
              child: Container(
                width: screenWidth * boxWidth,
                height: screenWidth * boxHeight,
                child: FlutterMap(
                  options: MapOptions(
                    center: LatLng(60.7666, 10.8471),
                    zoom: 9.0,
                  ),
                  children: [
                    TileLayer(
                      urlTemplate: "https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png",
                      subdomains: const ['a', 'b', 'c'],
                    ),
                    MarkerLayer(
                      markers: widget.markerList.map((MarkerTemplate markerTemplate) {
                        return Marker(
                          width: markerTemplate.size,
                          height: markerTemplate.size,
                          point: markerTemplate.location,
                          builder: (ctx) => GestureDetector(
                            onTap: () {
                              setState(() {
                                selectedMarker = markerTemplate;
                              });
                            },
                            child: Image.asset(
                              'assets/icons/circle-red.png',
                              color: markerTemplate.color,
                              width: markerTemplate.size,
                              height: markerTemplate.size,
                            ),
                          ),
                        );
                      }).toList(),
                    ),
                    Positioned(
                      bottom: 10,
                      right: 10,
                      child: ClipRRect(
                        borderRadius: BorderRadius.circular(10),
                        child: Container(
                          width: (screenWidth * boxWidth) / 2.3,
                          height: isMinimized ? 20 : (screenWidth * boxWidth) / 2.3,
                          color: darkBlue,
                          child: Stack(
                            children: [
                              Center(
                                child: Text(
                                  'Placeholder',
                                  style: regTextStyle,
                                ),
                              ),
                              Positioned(
                                top: isMinimized ? 0 : 5,
                                left: 5,
                                child: GestureDetector(
                                  onTap: () {
                                    setState(() {
                                      isMinimized = !isMinimized; // Toggle the minimized state
                                      print("IsMinimized?: $isMinimized");
                                    });
                                  },
                                  child: Icon(isMinimized ? Icons.arrow_upward : Icons.arrow_downward),
                                ),
                              ),
                            ],
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ),
            ),
            const SizedBox(height: contPadding), // Padding between containers
            ClipRRect(
              borderRadius: BorderRadius.circular(20),
              child: Container(
                width: screenWidth * boxWidth,
                height: screenWidth * boxHeight * 1.5, // NB: make dynamic
                child: Align(
                  alignment: Alignment.topLeft,
                  child: Padding(
                    padding: const EdgeInsets.only(top: 20, left: 20), // Edge padding, text
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                         Text(
                          'Ice stats',
                          style: titleStyle,
                        ),
                        Text(
                          'Safety level: ${selectedMarker?.geoData.safetyLevel}',
                          style: regTextStyle,
                        ),
                        Text(
                          'Location: (${selectedMarker?.geoData.latitude},${selectedMarker?.geoData.longitude})',
                          style: regTextStyle,
                        ),
                        const SizedBox(height: contPadding),
                        SizedBox(
                          width: screenWidth * boxWidth - contPadding * 2,
                          height: 200,
                          child: LineChart(
                            LineChartData(
                              backgroundColor: lightBlue,
                              titlesData: FlTitlesData(
                                leftTitles: SideTitles(showTitles: true),
                                bottomTitles: SideTitles(showTitles: true),
                              ),
                              borderData: FlBorderData(
                                show: true,
                              ),
                              minX: 0,
                              maxX: 4,
                              minY: 0,
                              maxY: 50,
                              lineBarsData: [
                                LineChartBarData(
                                  spots: chartData,
                                  isCurved: true,
                                  colors: [barBlue],
                                ),
                              ],
                            ),
                          ),
                        ),
                        const SizedBox(height: contPadding),
                        SizedBox(
                            width: screenWidth * boxWidth - contPadding * 2,
                            height: 160,
                            child: BarChart(
                              BarChartData(
                                alignment: BarChartAlignment.spaceAround,
                                maxY: 20,
                                barTouchData: BarTouchData(enabled: false),
                                titlesData: FlTitlesData(
                                  bottomTitles: SideTitles(
                                    showTitles: true,
                                    getTextStyles: (value) => chartTextStyle,
                                    margin: 10,
                                    getTitles: (value) {
                                      switch (value.toInt()) {
                                        case 0:
                                          return 'Placeholder1';
                                        case 1:
                                          return 'Placeholder2';
                                        case 2:
                                          return 'Placeholder3';
                                        default:
                                          return '';
                                      }
                                    },
                                  ),
                                  leftTitles: SideTitles(
                                    showTitles: true,
                                    getTextStyles: (value) => chartTextStyle,
                                    margin: 10,
                                    reservedSize: 30,
                                    interval: 5,
                                  ),
                                ),
                                borderData: FlBorderData(
                                  show: true,
                                  border: Border.all(color: Colors.white, width: 1),
                                ),
                                barGroups: [
                                  BarChartGroupData(
                                    x: 0,
                                    barRods: [
                                      BarChartRodData(
                                        y: 15, // Bar height
                                        width: barWidth, // Bar width
                                      ),
                                    ],
                                  ),
                                  BarChartGroupData(
                                    x: 1,
                                    barRods: [
                                      BarChartRodData(
                                        y: 10,
                                        width: barWidth,
                                      ),
                                    ],
                                  ),
                                  BarChartGroupData(
                                    x: 2,
                                    barRods: [
                                      BarChartRodData(
                                        y: 18,
                                        width: barWidth,
                                      ),
                                    ],
                                  ),
                                ],
                              ),
                            )
                        )
                      ],
                    ),
                  ),
                ),
              ),
            ),
          ],
        );
      },
    );
  }
}